"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.responses = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.middleware = middleware;

var _ajv = require("ajv");

var _ajv2 = _interopRequireDefault(_ajv);

var _ramda = require("ramda");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ajv = (0, _ajv2.default)({ allErrors: true });

function validateRequest(params, req) {
    return params.map(function (param) {
        // Extract the target to validate
        var target;
        if (param.in === "body") {
            target = req.body;
        }if (param.in === "path") {
            target = req.params[param.name];
        }if (param.in === "query") {
            target = req.query[param.name];
        }if (param.in === "header") {
            target = req.headers[param.name.toLowerCase()];
        }
        // Validate the target
        var valid;
        var error;
        if (param.required && target === undefined) {
            valid = false;
            error = {
                message: "Missing required parameter " + param.name + " in " + param.in
            };
        } else if (!param.required && target === undefined) {
            valid = true;
            error = null;
        } else {
            valid = param.validate(target);
            error = !valid ? {
                message: "Validation failed for parameter " + param.name + " in " + param.in,
                details: param.validate.errors
            } : null;
        }
        return { valid: valid, error: error };
    }).reduce(function (result, singleResult) {
        return {
            // Fail if any validation fails
            valid: result.valid && singleResult.valid,
            // Gather all validation errors
            errors: result.errors.concat(singleResult.error).filter(_ramda.identity)
        };
    }, { valid: true, errors: [] });
}

var responses = exports.responses = {
    "400": {
        description: "Validation failed"
    }
};
function middleware() {
    var params = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

    // Decorate each parameter with a validation function
    params = params.map(function (param) {
        return _extends({}, param, {
            validate: param.schema ? ajv.compile(param.schema) : _ramda.T
        });
    });
    // Return the actual middleware
    return function (req, res, next) {
        var result = validateRequest(params, req);
        if (result.valid) {
            next();
        } else {
            res.status(400).send({
                message: "Validation failed",
                errors: result.errors
            });
        }
    };
}